"""API configuration constants."""

# Session list limits
DEFAULT_SESSION_LIMIT = 50
MAX_SESSIONS_PER_PAGE = 1000

# Event/trace limits
DEFAULT_TRACE_LIMIT = 100
MAX_TRACES_PER_REQUEST = 10000
MAX_EVENTS_FOR_ANALYSIS = 1000

# Search limits
DEFAULT_SEARCH_LIMIT = 50
MAX_SEARCH_RESULTS = 500
MAX_SESSION_SEARCH_RESULTS = 100

# Alert limits
DEFAULT_ALERT_LIMIT = 50
MAX_ALERTS_PER_REQUEST = 200

# Text field limits
MAX_QUERY_LENGTH = 256
MIN_SEARCH_QUERY_LENGTH = 2
MAX_FIX_NOTE_LENGTH = 2000
